@@ -31,9 +31,7 @@ from sqlalchemy import types as sql_types
 from barbican.common import exception
 from barbican.common import utils
 from barbican import i18n as u
-from barbican.plugin.interface import secret_store
 
-LOG = utils.getLogger(__name__)
 BASE = declarative.declarative_base()
 ERROR_REASON_LENGTH = 255
 SUB_STATUS_LENGTH = 36
@@ -93,8 +91,8 @@ class ModelBase(object):
     """Base class for Nova and Barbican Models."""
     __table_args__ = {'mysql_engine': 'InnoDB'}
     __table_initialized__ = False
-    __protected_attributes__ = set([
-        "created_at", "updated_at", "deleted_at", "deleted"])
+    __protected_attributes__ = {
+        "created_at", "updated_at", "deleted_at", "deleted"}
 
     id = sa.Column(sa.String(36), primary_key=True,
                    default=utils.generate_uuid)
@@ -127,6 +125,9 @@ class ModelBase(object):
         self._do_delete_children(session)
         session.delete(self)
 
+    def get(self, key, default=None):
+        return getattr(self, key, default)
+
     def _do_delete_children(self, session):
         """Sub-class hook: delete children relationships."""
         pass
@@ -229,13 +230,22 @@ class ContainerSecret(BASE, SoftDeleteMixIn, ModelBase):
 
     # Eager load this relationship via 'lazy=False'.
     container = orm.relationship(
-        'Container', backref=orm.backref('container_secrets', lazy=False))
+        'Container',
+        backref=orm.backref('container_secrets', lazy=False,
+                            primaryjoin="and_(ContainerSecret.container_id == "
+                            "Container.id, ContainerSecret.deleted!=True)"))
     secrets = orm.relationship(
-        'Secret', backref=orm.backref('container_secrets'))
+        'Secret',
+        backref=orm.backref('container_secrets',
+                            primaryjoin="and_(ContainerSecret.secret_id == "
+                            "Secret.id, ContainerSecret.deleted!=True)"))
 
     __table_args__ = (sa.UniqueConstraint('container_id', 'secret_id', 'name',
                                           name='_container_secret_name_uc'),)
 
+    def __init__(self, check_exc=True):
+        super(ContainerSecret, self).__init__()
+
 
 class Project(BASE, SoftDeleteMixIn, ModelBase):
     """Represents a Project in the datastore.
@@ -254,6 +264,9 @@ class Project(BASE, SoftDeleteMixIn, ModelBase):
     cas = orm.relationship("ProjectCertificateAuthority", backref="project")
     project_quotas = orm.relationship("ProjectQuotas", backref="project")
 
+    def __init__(self, check_exc=True):
+        super(Project, self).__init__()
+
     def _do_extra_dict_fields(self):
         """Sub-class hook method: return dict of fields."""
         return {'external_id': self.external_id}
@@ -271,7 +284,7 @@ class Secret(BASE, SoftDeleteMixIn, ModelBase):
 
     name = sa.Column(sa.String(255))
     secret_type = sa.Column(sa.String(255),
-                            server_default=secret_store.SecretType.OPAQUE)
+                            server_default=utils.SECRET_TYPE_OPAQUE)
     expiration = sa.Column(sa.DateTime, default=None)
     algorithm = sa.Column(sa.String(255))
     bit_length = sa.Column(sa.Integer)
@@ -303,7 +316,12 @@ class Secret(BASE, SoftDeleteMixIn, ModelBase):
         backref="secret",
         cascade="all, delete-orphan")
 
-    def __init__(self, parsed_request=None):
+    consumers = orm.relationship(
+        "SecretConsumerMetadatum",
+        backref="secret",
+        cascade="all, delete-orphan")
+
+    def __init__(self, parsed_request=None, check_exc=True):
         """Creates secret from a dict."""
         super(Secret, self).__init__()
 
@@ -311,7 +329,7 @@ class Secret(BASE, SoftDeleteMixIn, ModelBase):
             self.name = parsed_request.get('name')
             self.secret_type = parsed_request.get(
                 'secret_type',
-                secret_store.SecretType.OPAQUE)
+                utils.SECRET_TYPE_OPAQUE)
             expiration = self._iso_to_datetime(parsed_request.get
                                                ('expiration'))
             self.expiration = expiration
@@ -356,6 +374,14 @@ class Secret(BASE, SoftDeleteMixIn, ModelBase):
             'bit_length': self.bit_length,
             'mode': self.mode,
             'creator_id': self.creator_id,
+            # TODO(redrobot): Uncomment this after adding microversions
+            # "consumers": [
+            #     {
+            #         "service": consumer.service,
+            #         "resource_type": consumer.resource_type,
+            #         "resource_id": consumer.resource_id,
+            #     } for consumer in self.consumers if not consumer.deleted
+            # ],
         }
 
 
@@ -369,17 +395,17 @@ class SecretStoreMetadatum(BASE, SoftDeleteMixIn, ModelBase):
     secret_id = sa.Column(
         sa.String(36), sa.ForeignKey('secrets.id'), index=True, nullable=False)
 
-    def __init__(self, key, value):
+    def __init__(self, key=None, value=None, check_exc=True):
         super(SecretStoreMetadatum, self).__init__()
 
         msg = u._("Must supply non-None {0} argument "
                   "for SecretStoreMetadatum entry.")
 
-        if key is None:
+        if key is None and check_exc:
             raise exception.MissingArgumentError(msg.format("key"))
         self.key = key
 
-        if value is None:
+        if value is None and check_exc:
             raise exception.MissingArgumentError(msg.format("value"))
         self.value = value
 
@@ -404,17 +430,17 @@ class SecretUserMetadatum(BASE, SoftDeleteMixIn, ModelBase):
     __table_args__ = (sa.UniqueConstraint('secret_id', 'key',
                                           name='_secret_key_uc'),)
 
-    def __init__(self, key, value):
+    def __init__(self, key=None, value=None, check_exc=True):
         super(SecretUserMetadatum, self).__init__()
 
         msg = u._("Must supply non-None {0} argument "
                   "for SecretUserMetadatum entry.")
 
-        if key is None:
+        if key is None and check_exc:
             raise exception.MissingArgumentError(msg.format("key"))
         self.key = key
 
-        if value is None:
+        if value is None and check_exc:
             raise exception.MissingArgumentError(msg.format("value"))
         self.value = value
 
@@ -445,7 +471,7 @@ class EncryptedDatum(BASE, SoftDeleteMixIn, ModelBase):
     # Eager load this relationship via 'lazy=False'.
     kek_meta_project = orm.relationship("KEKDatum", lazy=False)
 
-    def __init__(self, secret=None, kek_datum=None):
+    def __init__(self, secret=None, kek_datum=None, check_exc=True):
         """Creates encrypted datum from a secret and KEK metadata."""
         super(EncryptedDatum, self).__init__()
 
@@ -505,6 +531,9 @@ class KEKDatum(BASE, SoftDeleteMixIn, ModelBase):
     mode = sa.Column(sa.String(255))
     plugin_meta = sa.Column(sa.Text)
 
+    def __index__(self, check_exc=True):
+        super(KEKDatum, self).__init__()
+
     def _do_extra_dict_fields(self):
         """Sub-class hook method: return dict of fields."""
         return {'algorithm': self.algorithm}
@@ -554,17 +583,17 @@ class Order(BASE, SoftDeleteMixIn, ModelBase):
         backref="order",
         cascade="all, delete-orphan")
 
-    def __init__(self, parsed_request=None):
-            """Creates a Order entity from a dict."""
-            super(Order, self).__init__()
-            if parsed_request:
-                self.type = parsed_request.get('type')
-                self.meta = parsed_request.get('meta')
-                self.status = States.ACTIVE
-                self.sub_status = parsed_request.get('sub_status')
-                self.sub_status_message = parsed_request.get(
-                    'sub_status_message')
-                self.creator_id = parsed_request.get('creator_id')
+    def __init__(self, parsed_request=None, check_exc=True):
+        """Creates a Order entity from a dict."""
+        super(Order, self).__init__()
+        if parsed_request:
+            self.type = parsed_request.get('type')
+            self.meta = parsed_request.get('meta')
+            self.status = States.ACTIVE
+            self.sub_status = parsed_request.get('sub_status')
+            self.sub_status_message = parsed_request.get(
+                'sub_status_message')
+            self.creator_id = parsed_request.get('creator_id')
 
     def set_error_reason_safely(self, error_reason_raw):
         """Ensure error reason does not raise database attribute exceptions."""
@@ -625,17 +654,17 @@ class OrderPluginMetadatum(BASE, SoftDeleteMixIn, ModelBase):
     key = sa.Column(sa.String(255), nullable=False)
     value = sa.Column(sa.String(255), nullable=False)
 
-    def __init__(self, key, value):
+    def __init__(self, key=None, value=None, check_exc=True):
         super(OrderPluginMetadatum, self).__init__()
 
         msg = u._("Must supply non-None {0} argument "
                   "for OrderPluginMetadatum entry.")
 
-        if key is None:
+        if key is None and check_exc:
             raise exception.MissingArgumentError(msg.format("key"))
         self.key = key
 
-        if value is None:
+        if value is None and check_exc:
             raise exception.MissingArgumentError(msg.format("value"))
         self.value = value
 
@@ -661,17 +690,17 @@ class OrderBarbicanMetadatum(BASE, SoftDeleteMixIn, ModelBase):
     key = sa.Column(sa.String(255), nullable=False)
     value = sa.Column(sa.Text, nullable=False)
 
-    def __init__(self, key, value):
+    def __init__(self, key=None, value=None, check_exc=True):
         super(OrderBarbicanMetadatum, self).__init__()
 
         msg = u._("Must supply non-None {0} argument "
                   "for OrderBarbicanMetadatum entry.")
 
-        if key is None:
+        if key is None and check_exc:
             raise exception.MissingArgumentError(msg.format("key"))
         self.key = key
 
-        if value is None:
+        if value is None and check_exc:
             raise exception.MissingArgumentError(msg.format("value"))
         self.value = value
 
@@ -699,6 +728,9 @@ class OrderRetryTask(BASE, SoftDeleteMixIn, ModelBase):
     retry_kwargs = sa.Column(JsonBlob(), nullable=False)
     retry_count = sa.Column(sa.Integer, nullable=False, default=0)
 
+    def __index__(self, check_exc):
+        super(OrderRetryTask, self).__init__()
+
 
 class Container(BASE, SoftDeleteMixIn, ModelBase):
     """Represents a Container for Secrets in the datastore.
@@ -722,7 +754,7 @@ class Container(BASE, SoftDeleteMixIn, ModelBase):
     consumers = sa.orm.relationship("ContainerConsumerMetadatum")
     creator_id = sa.Column(sa.String(255))
 
-    def __init__(self, parsed_request=None):
+    def __init__(self, parsed_request=None, check_exc=True):
         """Creates a Container entity from a dict."""
         super(Container, self).__init__()
 
@@ -801,7 +833,8 @@ class ContainerConsumerMetadatum(BASE, SoftDeleteMixIn, ModelBase):
         sa.Index('values_index', 'container_id', 'name', 'URL')
     )
 
-    def __init__(self, container_id, project_id, parsed_request):
+    def __init__(self, container_id=None, project_id=None,
+                 parsed_request=None, check_exc=True):
         """Registers a Consumer to a Container."""
         super(ContainerConsumerMetadatum, self).__init__()
 
@@ -835,18 +868,18 @@ class TransportKey(BASE, SoftDeleteMixIn, ModelBase):
     plugin_name = sa.Column(sa.String(255), nullable=False)
     transport_key = sa.Column(sa.Text, nullable=False)
 
-    def __init__(self, plugin_name, transport_key):
-        """Creates transport key entity ."""
+    def __init__(self, plugin_name=None, transport_key=None, check_exc=True):
+        """Creates transport key entity."""
         super(TransportKey, self).__init__()
 
         msg = u._("Must supply non-None {0} argument for TransportKey entry.")
 
-        if plugin_name is None:
+        if plugin_name is None and check_exc:
             raise exception.MissingArgumentError(msg.format("plugin_name"))
 
         self.plugin_name = plugin_name
 
-        if transport_key is None:
+        if transport_key is None and check_exc:
             raise exception.MissingArgumentError(msg.format("transport_key"))
 
         self.transport_key = transport_key
@@ -882,8 +915,8 @@ class CertificateAuthority(BASE, ModelBase):
         cascade="all, delete-orphan"
     )
 
-    def __init__(self, parsed_ca_in):
-        """Creates certificate authority entity ."""
+    def __init__(self, parsed_ca_in=None, check_exc=True):
+        """Creates certificate authority entity."""
         super(CertificateAuthority, self).__init__()
 
         msg = u._("Must supply Non-None {0} argument "
@@ -957,17 +990,17 @@ class CertificateAuthorityMetadatum(BASE, ModelBase):
     __table_args__ = (sa.UniqueConstraint(
         'ca_id', 'key', name='_certificate_authority_metadatum_uc'),)
 
-    def __init__(self, key, value):
+    def __init__(self, key=None, value=None, check_exc=True):
         super(CertificateAuthorityMetadatum, self).__init__()
 
         msg = u._("Must supply non-None {0} argument "
                   "for CertificateAuthorityMetadatum entry.")
 
-        if key is None:
+        if key is None and check_exc:
             raise exception.MissingArgumentError(msg.format("key"))
         self.key = key
 
-        if value is None:
+        if value is None and check_exc:
             raise exception.MissingArgumentError(msg.format("value"))
         self.value = value
 
@@ -1003,18 +1036,18 @@ class ProjectCertificateAuthority(BASE, ModelBase):
     __table_args__ = (sa.UniqueConstraint(
         'project_id', 'ca_id', name='_project_certificate_authority_uc'),)
 
-    def __init__(self, project_id, ca_id):
+    def __init__(self, project_id=None, ca_id=None, check_exc=True):
         """Registers a Consumer to a Container."""
         super(ProjectCertificateAuthority, self).__init__()
 
         msg = u._("Must supply non-None {0} argument "
                   "for ProjectCertificateAuthority entry.")
 
-        if project_id is None:
+        if project_id is None and check_exc:
             raise exception.MissingArgumentError(msg.format("project_id"))
         self.project_id = project_id
 
-        if ca_id is None:
+        if ca_id is None and check_exc:
             raise exception.MissingArgumentError(msg.format("ca_id"))
         self.ca_id = ca_id
 
@@ -1055,18 +1088,18 @@ class PreferredCertificateAuthority(BASE, ModelBase):
     ca = orm.relationship('CertificateAuthority',
                           backref=orm.backref('preferred_ca'))
 
-    def __init__(self, project_id, ca_id):
+    def __init__(self, project_id=None, ca_id=None, check_exc=True):
         """Registers a Consumer to a Container."""
         super(PreferredCertificateAuthority, self).__init__()
 
         msg = u._("Must supply non-None {0} argument "
                   "for PreferredCertificateAuthority entry.")
 
-        if project_id is None:
+        if project_id is None and check_exc:
             raise exception.MissingArgumentError(msg.format("project_id"))
         self.project_id = project_id
 
-        if ca_id is None:
+        if ca_id is None and check_exc:
             raise exception.MissingArgumentError(msg.format("ca_id"))
         self.ca_id = ca_id
 
@@ -1109,18 +1142,18 @@ class SecretACL(BASE, ModelBase):
     __table_args__ = (sa.UniqueConstraint(
         'secret_id', 'operation', name='_secret_acl_operation_uc'),)
 
-    def __init__(self, secret_id, operation, project_access=None,
-                 user_ids=None):
+    def __init__(self, secret_id=None, operation=None, project_access=None,
+                 user_ids=None, check_exc=True):
         """Creates secret ACL entity."""
         super(SecretACL, self).__init__()
 
         msg = u._("Must supply non-None {0} argument for SecretACL entry.")
 
-        if secret_id is None:
+        if secret_id is None and check_exc:
             raise exception.MissingArgumentError(msg.format("secret_id"))
         self.secret_id = secret_id
 
-        if operation is None:
+        if operation is None and check_exc:
             raise exception.MissingArgumentError(msg.format("operation"))
         self.operation = operation
 
@@ -1185,18 +1218,18 @@ class ContainerACL(BASE, ModelBase):
     __table_args__ = (sa.UniqueConstraint(
         'container_id', 'operation', name='_container_acl_operation_uc'),)
 
-    def __init__(self, container_id, operation, project_access=None,
-                 user_ids=None):
+    def __init__(self, container_id=None, operation=None, project_access=None,
+                 user_ids=None, check_exc=True):
         """Creates container ACL entity."""
         super(ContainerACL, self).__init__()
 
         msg = u._("Must supply non-None {0} argument for ContainerACL entry.")
 
-        if container_id is None:
+        if container_id is None and check_exc:
             raise exception.MissingArgumentError(msg.format("container_id"))
         self.container_id = container_id
 
-        if operation is None:
+        if operation is None and check_exc:
             raise exception.MissingArgumentError(msg.format("operation"))
         self.operation = operation
 
@@ -1250,14 +1283,14 @@ class SecretACLUser(BASE, ModelBase):
     __table_args__ = (sa.UniqueConstraint(
         'acl_id', 'user_id', name='_secret_acl_user_uc'),)
 
-    def __init__(self, acl_id, user_id):
+    def __init__(self, acl_id=None, user_id=None, check_exc=True):
         """Creates secret ACL user entity."""
         super(SecretACLUser, self).__init__()
 
         msg = u._("Must supply non-None {0} argument for SecretACLUser entry.")
 
         self.acl_id = acl_id
-        if user_id is None:
+        if user_id is None and check_exc:
             raise exception.MissingArgumentError(msg.format("user_id"))
         self.user_id = user_id
         self.status = States.ACTIVE
@@ -1287,7 +1320,7 @@ class ContainerACLUser(BASE, ModelBase):
     __table_args__ = (sa.UniqueConstraint(
         'acl_id', 'user_id', name='_container_acl_user_uc'),)
 
-    def __init__(self, acl_id, user_id):
+    def __init__(self, acl_id=None, user_id=None, check_exc=True):
         """Creates container ACL user entity."""
         super(ContainerACLUser, self).__init__()
 
@@ -1295,7 +1328,7 @@ class ContainerACLUser(BASE, ModelBase):
                   "entry.")
 
         self.acl_id = acl_id
-        if user_id is None:
+        if user_id is None and check_exc:
             raise exception.MissingArgumentError(msg.format("user_id"))
         self.user_id = user_id
         self.status = States.ACTIVE
@@ -1327,7 +1360,8 @@ class ProjectQuotas(BASE, ModelBase):
     consumers = sa.Column(sa.Integer, nullable=True)
     cas = sa.Column(sa.Integer, nullable=True)
 
-    def __init__(self, project_id=None, parsed_project_quotas=None):
+    def __init__(self, project_id=None, parsed_project_quotas=None,
+                 check_exc=True):
         """Creates Project Quotas entity from a project and a dict.
 
         :param project_id: the internal id of the project with quotas
@@ -1340,7 +1374,7 @@ class ProjectQuotas(BASE, ModelBase):
 
         msg = u._("Must supply non-None {0} argument for ProjectQuotas entry.")
 
-        if project_id is None:
+        if project_id is None and check_exc:
             raise exception.MissingArgumentError(msg.format("project_id"))
         self.project_id = project_id
 
@@ -1373,3 +1407,162 @@ class ProjectQuotas(BASE, ModelBase):
         if self.cas:
             ret['cas'] = self.cas
         return ret
+
+
+class SecretStores(BASE, ModelBase):
+    """List of secret stores defined via service configuration.
+
+    This class provides a list of secret stores entities with their respective
+    secret store plugin and crypto plugin names.
+
+    SecretStores deletes are NOT soft-deletes.
+    """
+
+    __tablename__ = 'secret_stores'
+
+    store_plugin = sa.Column(sa.String(255), nullable=False)
+    crypto_plugin = sa.Column(sa.String(255), nullable=True)
+    global_default = sa.Column(sa.Boolean, nullable=False, default=False)
+    name = sa.Column(sa.String(255), nullable=False)
+
+    __table_args__ = (sa.UniqueConstraint(
+        'store_plugin', 'crypto_plugin',
+        name='_secret_stores_plugin_names_uc'),
+        sa.UniqueConstraint('name', name='_secret_stores_name_uc'),)
+
+    def __init__(self, name=None, store_plugin=None, crypto_plugin=None,
+                 global_default=None, check_exc=True):
+        """Creates secret store entity."""
+        super(SecretStores, self).__init__()
+
+        msg = u._("Must supply non-Blank {0} argument for SecretStores entry.")
+
+        if not name and check_exc:
+            raise exception.MissingArgumentError(msg.format("name"))
+        if not store_plugin and check_exc:
+            raise exception.MissingArgumentError(msg.format("store_plugin"))
+
+        self.store_plugin = store_plugin
+        self.name = name
+        self.crypto_plugin = crypto_plugin
+        if global_default is not None:
+            self.global_default = global_default
+
+        self.status = States.ACTIVE
+
+    def _do_extra_dict_fields(self):
+        """Sub-class hook method: return dict of fields."""
+        return {'secret_store_id': self.id,
+                'store_plugin': self.store_plugin,
+                'crypto_plugin': self.crypto_plugin,
+                'global_default': self.global_default,
+                'name': self.name}
+
+
+class ProjectSecretStore(BASE, ModelBase):
+    """Stores secret store to be used for new project secrets.
+
+    This class maintains secret store and project mapping so that new project
+    secret entries uses it as plugin backend.
+
+    ProjectSecretStores deletes are NOT soft-deletes.
+    """
+
+    __tablename__ = 'project_secret_store'
+
+    secret_store_id = sa.Column(sa.String(36),
+                                sa.ForeignKey('secret_stores.id'),
+                                index=True,
+                                nullable=False)
+    project_id = sa.Column(sa.String(36),
+                           sa.ForeignKey('projects.id'),
+                           index=True,
+                           nullable=False)
+
+    secret_store = orm.relationship("SecretStores", backref="project_store")
+    project = orm.relationship('Project',
+                               backref=orm.backref('preferred_secret_store'))
+
+    __table_args__ = (sa.UniqueConstraint(
+        'project_id', name='_project_secret_store_project_uc'),)
+
+    def __init__(self, project_id=None, secret_store_id=None, check_exc=True):
+        """Creates project secret store mapping entity."""
+        super(ProjectSecretStore, self).__init__()
+
+        msg = u._("Must supply non-None {0} argument for ProjectSecretStore "
+                  " entry.")
+
+        if not project_id and check_exc:
+            raise exception.MissingArgumentError(msg.format("project_id"))
+        self.project_id = project_id
+        if not secret_store_id and check_exc:
+            raise exception.MissingArgumentError(msg.format("secret_store_id"))
+        self.secret_store_id = secret_store_id
+
+        self.status = States.ACTIVE
+
+    def _do_extra_dict_fields(self):
+        """Sub-class hook method: return dict of fields."""
+        return {'secret_store_id': self.secret_store_id,
+                'project_id': self.project_id}
+
+
+class SecretConsumerMetadatum(BASE, SoftDeleteMixIn, ModelBase):
+    """Stores Consumer Registrations for Secrets in the datastore.
+
+    Services can register interest in Secrets. Services will provide a
+    resource type and a resource id for the object that is using the Secret.
+    """
+
+    __tablename__ = "secret_consumer_metadata"
+
+    secret_id = sa.Column(
+        sa.String(36), sa.ForeignKey("secrets.id"), index=True, nullable=False
+    )
+    project_id = sa.Column(
+        sa.String(36), sa.ForeignKey("projects.id"), index=True, nullable=True
+    )
+    service = sa.Column(sa.String(255), nullable=False)
+    resource_type = sa.Column(sa.String(255), nullable=False)
+    resource_id = sa.Column(sa.String(36), index=True, nullable=False)
+
+    __table_args__ = (
+        sa.UniqueConstraint(
+            "secret_id", "resource_id", name="_secret_consumer_resource_uc"
+        ),
+    )
+
+    def __init__(self, secret_id=None, project_id=None, service=None,
+                 resource_type=None, resource_id=None, check_exc=True):
+        """Registers a Consumer to a Secret."""
+        super(SecretConsumerMetadatum, self).__init__()
+
+        msg = u._("Must supply non-None {0} argument "
+                  "for SecretConsumerMetadatum entry.")
+
+        if secret_id is None and check_exc:
+            raise exception.MissingArgumentError(msg.format("secret_id"))
+        if project_id is None and check_exc:
+            raise exception.MissingArgumentError(msg.format("project_id"))
+        if service is None and check_exc:
+            raise exception.MissingArgumentError(msg.format("service"))
+        if resource_type is None and check_exc:
+            raise exception.MissingArgumentError(msg.format("resource_type"))
+        if resource_id is None and check_exc:
+            raise exception.MissingArgumentError(msg.format("resource_id"))
+
+        self.secret_id = secret_id
+        self.project_id = project_id
+        self.service = service
+        self.resource_type = resource_type
+        self.resource_id = resource_id
+        self.status = States.ACTIVE
+
+    def _do_extra_dict_fields(self):
+        """Sub-class hook method: return dict of fields."""
+        return {
+            "service": self.service,
+            "resource_type": self.resource_type,
+            "resource_id": self.resource_id,
+        }
